/* 
 * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 * Copyright (C) 2005/2006, Anthony Minessale II <anthmct@yahoo.com>
 *
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 *
 * The Initial Developer of the Original Code is
 * Anthony Minessale II <anthmct@yahoo.com>
 * Portions created by the Initial Developer are Copyright (C)
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Anthony Minessale II <anthmct@yahoo.com>
 *
 *
 * mod_zaptimer.c -- Zaptel Timer Module
 *
 */
#include <switch.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#ifdef __linux__
#include <linux/zaptel.h>
#else
#include <zaptel.h>
#endif /* __linux__ */


static const char modname[] = "mod_zaptimer";

struct zap_timer_private {
	int flags;
	int fd;
	int samples;
};

static switch_status zap_timer_init(switch_timer *timer)
{
	struct zap_timer_private *private;

	if ((private = switch_core_alloc(timer->memory_pool, sizeof(*private)))) {
		private->flags = 1;
		if ((private->fd = open("/dev/zap/timer", O_RDWR)) > -1) {
			int samples;
			private->samples = timer->interval * 8;
			samples = private->samples;
			timer->private = private;
			ioctl(private->fd, ZT_TIMERCONFIG, &samples);
			return SWITCH_STATUS_SUCCESS;
		}
	}

	return SWITCH_STATUS_GENERR;
}

static switch_status zap_timer_next(switch_timer *timer)
{
	struct zap_timer_private *private = timer->private;
	int zt;


	if (switch_socket_waitfor(private->fd, -1, POLLPRI|POLLERR) < 0) {
		return SWITCH_STATUS_GENERR;
	}

	if (ioctl(private->fd, ZT_TIMERACK, &zt)) {
		return SWITCH_STATUS_GENERR;
	}

	timer->samplecount += timer->samples;

	return SWITCH_STATUS_SUCCESS; 
}

static switch_status zap_timer_destroy(switch_timer *timer)
{
	struct zap_timer_private *private = timer->private;

	if (private->fd > -1) {
		close(private->fd);
		private->fd = -1;
	}
	timer->private = NULL;
	return SWITCH_STATUS_SUCCESS;
}

static const switch_timer_interface zap_timer_interface = {
	/*.interface_name*/		"zap",
	/*.timer_init*/			zap_timer_init,
	/*.timer_next*/			zap_timer_next,
	/*.timer_destroy*/		zap_timer_destroy
};

static const switch_loadable_module_interface mod_timers_module_interface = {
	/*.module_name = */ modname,
	/*.endpoint_interface = */ NULL,
	/*.timer_interface = */ &zap_timer_interface,
	/*.dialplan_interface = */ NULL,
	/*.codec_interface = */ NULL,
	/*.application_interface =*/ NULL
};

Public switch_status switch_module_load(const switch_loadable_module_interface **interface) {
	
	/* connect my internal structure to the blank pointer passed to me */
	*interface = &mod_timers_module_interface;

	/* indicate that the module should continue to be loaded */
	return SWITCH_STATUS_SUCCESS;
}

